# 
# Project automation script for BrainCPLD 
# 
# Created for ISE version 10.1
# 
# This file contains several Tcl procedures (procs) that you can use to automate
# your project by running from xtclsh or the Project Navigator Tcl console.
# If you load this file (using the Tcl command: source BrainCPLD.tcl, then you can
# run any of the procs included here.
# You may also edit any of these procs to customize them. See comments in each
# proc for more instructions.
# 
# This file contains the following procedures:
# 
# Top Level procs (meant to be called directly by the user):
#    run_process: you can use this top-level procedure to run any processes
#        that you choose to by adding and removing comments, or by
#        adding new entries.
#    rebuild_project: you can alternatively use this top-level procedure
#        to recreate your entire project, and the run selected processes.
# 
# Lower Level (helper) procs (called under in various cases by the top level procs):
#    show_help: print some basic information describing how this script works
#    add_source_files: adds the listed source files to your project.
#    set_project_props: sets the project properties that were in effect when this
#        script was generated.
#    create_libraries: creates and adds file to VHDL libraries that were defined when
#        this script was generated.
#    create_partitions: adds any partitions that were defined when this script was generated.
#    set_process_props: set the process properties as they were set for your project
#        when this script was generated.
# 

set myProject "UltiMem.xise"
set myScript "BrainCPLD.tcl"

# 
# Main (top-level) routines
# 

# 
# run_process
# This procedure is used to run processes on an existing project. You may comment or
# uncomment lines to control which processes are run. This routine is set up to run
# the Implement Design and Generate Programming File processes by default. This proc
# also sets process properties as specified in the "set_process_props" proc. Only
# those properties which have values different from their current settings in the project
# file will be modified in the project.
# 
proc run_process {} {

   global myScript
   global myProject

   ## put out a 'heartbeat' - so we know something's happening.
   puts "\n$myScript: running ($myProject)...\n"

   if { ! [ open_project ] } {
      return false
   }

   set_process_props
   #
   # Remove the comment characters (#'s) to enable the following commands 
   # process run "Synthesize"
   # process run "Translate"
   # process run "Map"
   # process run "Place & Route"
   #
   puts "Running 'Implement Design'"
   if { ! [ process run "Implement Design" ] } {
      puts "$myScript: Implementation run failed, check run output for details."
      project close
      return
   }
   puts "Running 'Generate Programming File'"
   if { ! [ process run "Generate Programming File" ] } {
      puts "$myScript: Generate Programming File run failed, check run output for details."
      project close
      return
   }

   puts "Run completed."
   project close

}

# 
# rebuild_project
# 
# This procedure renames the project file (if it exists) and recreates the project.
# It then sets project properties and adds project sources as specified by the
# set_project_props and add_source_files support procs. It recreates VHDL libraries
# and partitions as they existed at the time this script was generated.
# 
# It then calls run_process to set process properties and run selected processes.
# 
proc rebuild_project {} {

   global myScript
   global myProject

   ## put out a 'heartbeat' - so we know something's happening.
   puts "\n$myScript: rebuilding ($myProject)...\n"

   if { [ file exists $myProject ] } { 
      puts "$myScript: Removing existing project file."
      file delete $myProject
   }

   puts "$myScript: Rebuilding project $myProject"
   project new $myProject
   set_project_props
   add_source_files
   create_libraries
   create_partitions
   puts "$myScript: project rebuild completed."

   run_process

}

# 
# Support Routines
# 

# 
# show_help: print information to help users understand the options available when
#            running this script.
# 
proc show_help {} {

   global myScript

   puts ""
   puts "usage: xtclsh $myScript <options>"
   puts "       or you can run xtclsh and then enter 'source $myScript'."
   puts ""
   puts "options:"
   puts "   run_process       - set properties and run processes."
   puts "   rebuild_project   - rebuild the project from scratch and run processes."
   puts "   set_project_props - set project properties (device, speed, etc.)"
   puts "   add_source_files  - add source files"
   puts "   create_libraries  - create vhdl libraries"
   puts "   create_partitions - create partitions"
   puts "   set_process_props - set process property values"
   puts "   show_help         - print this message"
   puts ""
}

proc open_project {} {

   global myScript
   global myProject

   if { ! [ file exists $myProject ] } { 
      ## project file isn't there, rebuild it.
      puts "Project $myProject not found. Use project_rebuild to recreate it."
      return false
   }

   project open $myProject

   return true

}
# 
# set_project_props
# 
# This procedure sets the project properties as they were set in the project
# at the time this script was generated.
# 
proc set_project_props {} {

   global myScript

   if { ! [ open_project ] } {
      return false
   }

   puts "$myScript: Setting project properties..."

   project set family "XC9500XL CPLDs"
   project set device "xc9572xl"
   project set package "TQ100"
   project set speed "-5"
   project set top_level_module_type "HDL"
   project set synthesis_tool "XST (VHDL/Verilog)"
   project set simulator "ISE Simulator (VHDL/Verilog)"
   project set "Preferred Language" "Verilog"
   project set "Enable Message Filtering" "true"
   project set "Display Incremental Messages" "true"

}


# 
# add_source_files
# 
# This procedure add the source files that were known to the project at the
# time this script was generated.
# 
proc add_source_files {} {

   global myScript

   if { ! [ open_project ] } {
      return false
   }

   puts "$myScript: Adding sources to project..."

   xfile add "UltiMem.v"
   xfile add "comparator.v"
   xfile add "decoder.v"
   xfile add "encoder.v"
   xfile add "mux.v"
   xfile add "register.v"

   # Set the Top Module as well...
   project set top "UltiMem"

   puts "$myScript: project sources reloaded."

} ; # end add_source_files

# 
# create_libraries
# 
# This procedure defines VHDL libraries and associates files with those libraries.
# It is expected to be used when recreating the project. Any libraries defined
# when this script was generated are recreated by this procedure.
# 
proc create_libraries {} {

   global myScript

   if { ! [ open_project ] } {
      return false
   }

   puts "$myScript: Creating libraries..."
   # note: if you have multiple files with the same name at different paths,
   # you may have problems with the lib_vhdl command.


   # must close the project or library definitions aren't saved, then reopen it for further use.
   project close
   open_project

} ; # end create_libraries

#
# create_partitions
#
# This procedure creates partitions on instances in your project.
# It is expected to be used when recreating the project. Any partitions
# defined when this script was generated are recreated by this procedure.
# 
proc create_partitions {} {

   global myScript

   if { ! [ open_project ] } {
      return false
   }

   puts "$myScript: Creating Partitions..."


   # must close the project or partition definitions aren't saved, then reopen it for further use.
   project close
   open_project

} ; # end create_partitions

# 
# set_process_props
# 
# This procedure sets properties as requested during script generation (either
# all of the properties, or only those modified from their defaults).
# 
proc set_process_props {} {

   global myScript

   if { ! [ open_project ] } {
      return false
   }

   puts "$myScript: setting process properties..."

   project set "Compiled Library Directory" "\$XILINX/<language>/<simulator>"
   project set "Use SmartGuide" "false"
   project set "SmartGuide Filename" "UltiMem_guide.ncd"
   project set "Preserve Unused Inputs" "false" -process "Fit"
   project set "I/O Pin Termination" "Keeper" -process "Fit"
   project set "VCCIO Reference Voltage" "LVTTL" -process "Generate IBIS Model"
   project set "WYSIWYG" "None" -process "Synthesize - XST"
   project set "WYSIWYG" "None" -process "Compile Logic"
   project set "WYSIWYG" "None" -process "Compile Design"
   project set "WYSIWYG" "None" -process "Optimize Hierarchy"
   project set "Library Search Order File" "" -process "Simulate Post-Fit Model"
   project set "Other Compiler Options" "" -process "Simulate Post-Fit Model"
   project set "Delay Values To Be Read from SDF" "Setup Time" -process "Simulate Post-Fit Model"
   project set "Use Custom Simulation Command File" "false" -process "Simulate Post-Fit Model"
   project set "Use Custom Simulation Command File" "false" -process "Simulate Post-Fit Model"
   project set "ISim UUT Instance Name" "UUT" -process "Simulate Post-Fit Model"
   project set "iMPACT Project File" "Default" -process "Generate SVF/XSVF/STAPL File"
   project set "Delay Values To Be Read from SDF" "Setup Time" -process "Simulate Post-Fit Model"
   project set "Clock Enable" "true" -process "Synthesize - XST"
   project set "Clock Enable" "true" -process "Compile Design"
   project set "Clock Enable" "true" -process "Optimize Hierarchy"
   project set "Use Automatic Do File" "true" -process "Simulate Post-Fit Model"
   project set "Generate Testbench File" "false" -process "Simulate Post-Fit Model"
   project set "Generate Testbench File" "false" -process "Generate Post-Fit Simulation Model"
   project set "Other NETGEN Command Line Options" "" -process "Simulate Post-Fit Model"
   project set "Other NETGEN Command Line Options" "" -process "Generate Post-Fit Simulation Model"
   project set "Retain Hierarchy" "true" -process "Simulate Post-Fit Model"
   project set "Retain Hierarchy" "true" -process "Generate Post-Fit Simulation Model"
   project set "Use Custom Do File" "false" -process "Simulate Post-Fit Model"
   project set "Case Implementation Style" "None" -process "Synthesize - XST"
   project set "Mux Extraction" "Yes" -process "Synthesize - XST"
   project set "Mux Extraction" "Yes" -process "Compile Design"
   project set "Mux Extraction" "Yes" -process "Optimize Hierarchy"
   project set "FSM Encoding Algorithm" "Auto" -process "Synthesize - XST"
   project set "Optimization Goal" "Speed" -process "Synthesize - XST"
   project set "Optimization Goal" "Speed" -process "Compile Design"
   project set "Optimization Goal" "Speed" -process "Optimize Hierarchy"
   project set "Optimization Effort" "Normal" -process "Synthesize - XST"
   project set "Optimization Effort" "Normal" -process "Compile Design"
   project set "Optimization Effort" "Normal" -process "Optimize Hierarchy"
   project set "Resource Sharing" "false" -process "Synthesize - XST"
   project set "Exhaustive Fit Mode" "false" -process "Fit"
   project set "HDL Equations Style" "Source" -process "Fit"
   project set "Other CPLD Fitter Command Line Options" "" -process "Fit"
   project set "Generate Post-Fit Simulation Model" "false" -process "Fit"
   project set "Generate Post-Fit Simulation Model" "false" -process "Simulate Post-Fit Model"
   project set "Autosignature Generation" "false" -process "Generate Programming File"
   project set "Other Programming Command Line Options" "" -process "Generate Programming File"
   project set "Show All Models" "false" -process "Generate IBIS Model"
   project set "Compile for HDL Debugging" "true" -process "Simulate Post-Fit Model"
   project set "Incremental Compilation" "true" -process "Simulate Post-Fit Model"
   project set "Specify 'define Macro Name and Value" "" -process "Simulate Post-Fit Model"
   project set "Specify Search Directories for 'Include" "" -process "Simulate Post-Fit Model"
   project set "Value Range Check" "false" -process "Simulate Post-Fit Model"
   project set "Target UCF File Name" "" -process "Lock Pins"
   project set "Other Ngdbuild Command Line Options" "" -process "Translate"
   project set "Other Timing Report Command Line Options" "" -process "Fit"
   project set "Other Timing Report Command Line Options" "" -process "Generate Timing"
   project set "Create Programmable GND Pins on Unused I/O" "false" -process "Fit"
   project set "Default Powerup Value of Registers" "Low" -process "Fit"
   project set "Collapsing Input Limit (2-54)" "28" -process "Fit"
   project set "Use Multi-level Logic Optimization" "true" -process "Fit"
   project set "Macrocell Power Setting" "Std" -process "Fit"
   project set "Output Slew Rate" "Fast" -process "Fit"
   project set "Use Timing Constraints" "true" -process "Fit"
   project set "Implementation Template" "Optimize Speed" -process "Fit"
   project set "Timing Report Format" "Summary" -process "Fit"
   project set "Timing Report Format" "Summary" -process "Generate Timing"
   project set "Use Global Clocks" "true" -process "Fit"
   project set "Use Global Output Enables" "true" -process "Fit"
   project set "Use Global Set/Reset" "true" -process "Fit"
   project set "Use Location Constraints" "Always" -process "Fit"
   project set "Create IEEE 1532 Configuration File" "false" -process "Generate Programming File"
   project set "Macro Search Path" "" -process "Translate"
   project set "Allow Unmatched LOC Constraints" "false" -process "Translate"
   project set "Add I/O Buffers" "true" -process "Synthesize - XST"
   project set "Add I/O Buffers" "true" -process "Compile Design"
   project set "Add I/O Buffers" "true" -process "Optimize Hierarchy"
   project set "Keep Hierarchy" "Yes" -process "Synthesize - XST"
   project set "Keep Hierarchy" "Yes" -process "Compile Design"
   project set "Keep Hierarchy" "Yes" -process "Optimize Hierarchy"
   project set "Macro Preserve" "false" -process "Synthesize - XST"
   project set "Macro Preserve" "false" -process "Compile Design"
   project set "Macro Preserve" "false" -process "Optimize Hierarchy"
   project set "XOR Preserve" "false" -process "Synthesize - XST"
   project set "XOR Preserve" "false" -process "Compile Design"
   project set "XOR Preserve" "false" -process "Optimize Hierarchy"
   project set "Bus Delimiter" "<>" -process "Synthesize - XST"
   project set "Case" "Maintain" -process "Synthesize - XST"
   project set "Equivalent Register Removal" "false" -process "Synthesize - XST"
   project set "Generate RTL Schematic" "Yes" -process "Synthesize - XST"
   project set "Generics, Parameters" "" -process "Synthesize - XST"
   project set "Hierarchy Separator" "/" -process "Synthesize - XST"
   project set "HDL INI File" "" -process "Synthesize - XST"
   project set "Library Search Order" "" -process "Synthesize - XST"
   project set "Netlist Hierarchy" "As Optimized" -process "Synthesize - XST"
   project set "Use Synthesis Constraints File" "true" -process "Synthesize - XST"
   project set "Custom Compile File List" "" -process "Synthesize - XST"
   project set "Verilog Include Directories" "" -process "Synthesize - XST"
   project set "Verilog 2001" "true" -process "Synthesize - XST"
   project set "Verilog Macros" "" -process "Synthesize - XST"
   project set "Work Directory" "./xst" -process "Synthesize - XST"
   project set "Other XST Command Line Options" "" -process "Synthesize - XST"
   project set "Logic Optimization" "Speed" -process "Fit"
   project set "Custom Simulation Command File" "" -process "Simulate Post-Fit Model"
   project set "Custom Simulation Command File" "" -process "Simulate Post-Fit Model"
   project set "Generate VCD File for Power Estimation" "false" -process "Simulate Post-Fit Model"
   project set "Generate VCD File for Power Estimation" "false" -process "Simulate Post-Fit Model"
   project set "Run for Specified Time" "true" -process "Simulate Post-Fit Model"
   project set "Run for Specified Time" "true" -process "Simulate Post-Fit Model"
   project set "Data Flow window" "false" -process "Simulate Post-Fit Model"
   project set "List window" "false" -process "Simulate Post-Fit Model"
   project set "Process window" "false" -process "Simulate Post-Fit Model"
   project set "Signal window" "true" -process "Simulate Post-Fit Model"
   project set "Simulation Resolution" "Default (1 ps)" -process "Simulate Post-Fit Model"
   project set "Simulation Run Time" "1000ns" -process "Simulate Post-Fit Model"
   project set "Simulation Run Time" "1000ns" -process "Simulate Post-Fit Model"
   project set "Source window" "false" -process "Simulate Post-Fit Model"
   project set "Structure window" "true" -process "Simulate Post-Fit Model"
   project set "ModelSim Post-Fit UUT Instance Name" "UUT" -process "Simulate Post-Fit Model"
   project set "Variables window" "false" -process "Simulate Post-Fit Model"
   project set "Wave window" "true" -process "Simulate Post-Fit Model"
   project set "Custom Do File" "" -process "Simulate Post-Fit Model"
   project set "Generate VCD File" "false" -process "Simulate Post-Fit Model"
   project set "Rename Design Instance in Testbench File to" "UUT" -process "Simulate Post-Fit Model"
   project set "Rename Design Instance in Testbench File to" "UUT" -process "Generate Post-Fit Simulation Model"
   project set "VHDL Syntax" "93" -process "Simulate Post-Fit Model"
   project set "Use Explicit Declarations Only" "true" -process "Simulate Post-Fit Model"
   project set "Synthesis Constraints File" "" -process "Synthesize - XST"
   project set "Other VCOM Command Line Options" "" -process "Simulate Post-Fit Model"
   project set "Other VLOG Command Line Options" "" -process "Simulate Post-Fit Model"
   project set "Other VSIM Command Line Options" "" -process "Simulate Post-Fit Model"
   project set "Signature /User Code" "" -process "Generate Programming File"
   project set "Collapsing Pterm Limit (1-90)" "5" -process "Fit"
   project set "Safe Implementation" "No" -process "Synthesize - XST"
   project set "Safe Implementation" "No" -process "Compile Design"
   project set "Simulation Run Time" "1000 ns" -process "Simulate Post-Fit Model"
   project set "Simulation Run Time" "1000 ns" -process "Simulate Post-Fit Model"
   project set "VCD File Name" "xpower.vcd" -process "Simulate Post-Fit Model"
   project set "VCD File Name" "xpower.vcd" -process "Simulate Post-Fit Model"
   project set "Simulation Model Target" "Verilog" -process "Simulate Post-Fit Model"
   project set "Simulation Model Target" "Verilog" -process "Generate Post-Fit Simulation Model"
   project set "Testbench Model Target Language" "Verilog" -process "Simulate Post-Fit Model"
   project set "Automatically Insert glbl Module in the Netlist" "true" -process "Simulate Post-Fit Model"
   project set "Automatically Insert glbl Module in the Netlist" "true" -process "Generate Post-Fit Simulation Model"
   project set "Generate Architecture Only (No Entity Declaration)" "false" -process "Simulate Post-Fit Model"
   project set "Generate Architecture Only (No Entity Declaration)" "false" -process "Generate Post-Fit Simulation Model"
   project set "Include sdf_annotate task in Verilog File" "true" -process "Simulate Post-Fit Model"
   project set "Include sdf_annotate task in Verilog File" "true" -process "Generate Post-Fit Simulation Model"
   project set "Include SIMPRIM Models in Verilog File" "false" -process "Simulate Post-Fit Model"
   project set "Include SIMPRIM Models in Verilog File" "false" -process "Generate Post-Fit Simulation Model"
   project set "Include 'uselib Directive in Verilog File" "false" -process "Simulate Post-Fit Model"
   project set "Include 'uselib Directive in Verilog File" "false" -process "Generate Post-Fit Simulation Model"
   project set "Do Not Escape Signal and Instance Names in Netlist" "false" -process "Simulate Post-Fit Model"
   project set "Do Not Escape Signal and Instance Names in Netlist" "false" -process "Generate Post-Fit Simulation Model"
   project set "Output Extended Identifiers" "false" -process "Simulate Post-Fit Model"
   project set "Output Extended Identifiers" "false" -process "Generate Post-Fit Simulation Model"
   project set "Rename Top Level Architecture To" "Structure" -process "Simulate Post-Fit Model"
   project set "Rename Top Level Architecture To" "Structure" -process "Generate Post-Fit Simulation Model"
   project set "Rename Top Level Module To" "" -process "Simulate Post-Fit Model"
   project set "Rename Top Level Module To" "" -process "Generate Post-Fit Simulation Model"
   project set "Bring Out Global Set/Reset Net as a Port" "false" -process "Simulate Post-Fit Model"
   project set "Bring Out Global Set/Reset Net as a Port" "false" -process "Generate Post-Fit Simulation Model"
   project set "Path Used in sdf_annotate task" "Default" -process "Simulate Post-Fit Model"
   project set "Path Used in sdf_annotate task" "Default" -process "Generate Post-Fit Simulation Model"
   project set "Rename Top Level Entity to" "" -process "Simulate Post-Fit Model"
   project set "Rename Top Level Entity to" "" -process "Generate Post-Fit Simulation Model"
   project set "Global Set/Reset Port Name" "GSR_PORT" -process "Simulate Post-Fit Model"
   project set "Global Set/Reset Port Name" "GSR_PORT" -process "Generate Post-Fit Simulation Model"
   project set "Reset On Configuration Pulse Width" "100" -process "Simulate Post-Fit Model"
   project set "Reset On Configuration Pulse Width" "100" -process "Generate Post-Fit Simulation Model"

   puts "$myScript: project property values set."

} ; # end set_process_props

proc main {} {

   if { [llength $::argv] == 0 } {
      show_help
      return true
   }

   foreach option $::argv {
      switch $option {
         "show_help"           { show_help }
         "run_process"         { run_process }
         "rebuild_project"     { rebuild_project }
         "set_project_props"   { set_project_props }
         "add_source_files"    { add_source_files }
         "create_libraries"    { create_libraries }
         "create_partitions"   { create_partitions }
         "set_process_props"   { set_process_props }
         default               { puts "unrecognized option: $option"; show_help }
      }
   }
}

if { $tcl_interactive } {
   show_help
} else {
   if {[catch {main} result]} {
      puts "$myScript failed: $result."
   }
}

